@@ -74,7 +74,8 @@ EVALUATION_PATH = 'evaluation'
 
 # Resource Ids patterns
 SOURCE_RE = re.compile(r'^%s/[a-f,0-9]{24}$' % SOURCE_PATH)
-DATASET_RE = re.compile(r'^%s/[a-f,0-9]{24}$' % DATASET_PATH)
+DATASET_RE = re.compile(r'^%s/[a-f,0-9]{24}$|^public/%s/[a-f,0-9]{24}$' %
+                        (DATASET_PATH, DATASET_PATH))
 MODEL_RE = re.compile(r'^%s/[a-f,0-9]{24}$|^public/%s/[a-f,0-9]{24}$' %
                       (MODEL_PATH, MODEL_PATH))
 PREDICTION_RE = re.compile(r'^%s/[a-f,0-9]{24}$' % PREDICTION_PATH)
@@ -187,6 +188,8 @@ def get_status(resource):
     if not isinstance(resource, dict):
         raise ValueError("We need a complete resource to extract its status")
     if 'object' in resource:
+        if resource['object'] is None:
+            raise ValueError("The resource has no status info\n%s" % resource)
         resource = resource['object']
     if not resource.get('private', True):
         status = {'code': FINISHED}
@@ -775,7 +778,7 @@ class BigML(object):
             'object': resource,
             'error': error}
 
-    def _upload_source(self, url, args, source, out=sys.stdout):
+    def _upload_source(self, args, source, out=sys.stdout):
         """Uploads a source asynchronously.
 
         """
@@ -844,7 +847,7 @@ class BigML(object):
                                       'code': UPLOADING,
                                       'progress': 0.0}},
                 'error': error}
-            upload_args = (self.source_url + self.auth, args, source)
+            upload_args = (args, source)
             thread = Thread(target=self._upload_source,
                             args=upload_args,
                             kwargs={'out': out})
